@@ -1,3 +1,22 @@
+# LICENSE
+
+#Copyright (c) 2020 Cristiano Sansò
+
+#Permission is hereby granted, free of charge,
+#to any person obtaining a copy of this software and associated documentation files (the "Software"),
+#to deal in the Software without restriction, including without limitation the rights to
+#use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
+#and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
+
+#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
+
+#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
+#INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+#IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+#CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+
 # variables
 __pack = 'htam'
 
@@ -18,11 +37,11 @@ def __print_info(func, ops = ''):
     else:
         b1 = b2 = int((a - len(__pack + '.' + func + o))/2) - 1
     
-    print('\n' + '|============= INFO =============|')
-    print('|' + ' '*(a-2) + '|')
-    print('|' + ' '*b1 + __pack + '.' + func + o + ' '*b2 + '|' + '')
-    print('|' + ' '*(a-2) + '|')
-    print('|================================|')
+    print('\n' + '=============== INFO ===============')
+    print('[]' + ' '*(a-2) + '[]')
+    print('[]' + ' '*b1 + __pack + '.' + func + o + ' '*b2 + '[]')
+    print('[]' + ' '*(a-2) + '[]')
+    print('====================================')
 
 
 
@@ -31,13 +50,7 @@ def floor(x):
     '''\nthis function returns the greatest integer less or equal to x\nit returns "None" if the argument is not valid'''
 
     try:
-        if x >= 0:
-            return int(x)
-        else:
-            if int(x) == x:
-                return int(x)
-            else:
-                return int(x) - 1
+        return int(x // 1)
     except:
         return None
 
@@ -51,10 +64,7 @@ def ceil(x):
         if int(x) == x:
             return int(x)
         else:
-            if x >= 0:
-                return int(x) + 1
-            else:
-                return int(x)
+            return int(x // 1) + 1
     except:
         return None
 
@@ -89,56 +99,6 @@ def root(x, y = 2):
 
 
 
-# Mod
-def mod(x, y = 10, z = 0):
-    '''\ny is an integer automatically set to 10\nz is an integer automatically set to 0\n
-        \nthis function returns the solution to the equation
-        \n      zk ≡ x (mod y)
-        \nthe result will be k = "solution" (mod y) with "solution" returned
-        \nthis function will return "None" if the equation has no solution, or if 1 or more arguments are not valid
-        \nif z is missing or set to 0 this fuction will return
-        \n        x (mod y)'''
-    
-    try:
-        while True:
-            z0 = z
-            y0 = y
-            while y > 0:
-                z, y = y, z % y
-                gcd_xy = z
-            
-            z = int(z0)
-            y = int(y0)
-
-            if gcd_xy == 1:
-                if z == 0:
-                    return x % y
-                else:
-                    inverse = 0
-                    z = z % y
-                    x = x % y
-                    for i in range(1, y):
-                        if z*i % y == 1:
-                            inverse = i
-                            break
-                        else:
-                            continue
-                    return int((x*inverse) % y)
-            else:
-                try:
-                    assert int(x/gcd_xy) == x/gcd_xy
-                    x /= gcd_xy
-                    z /= gcd_xy
-                    y /= gcd_xy
-                    continue
-                except:
-                    return None
-    except:
-        return None
-    
-
-
-
 # GCD
 def gcd(x, y):
     '''this function returns the greatest common divisor of two numbers\nit returns "None" if 1 or more arguments are not valid'''
@@ -149,7 +109,7 @@ def gcd(x, y):
         return x
     except:
         return None
-    
+
 
 
 # LCM
@@ -174,6 +134,47 @@ def lcm(x, y):
 
 
 
+# Mod
+def mod(x, y = 10, z = 0):
+    '''\ny is an integer automatically set to 10\nz is an integer automatically set to 0\n
+        \nthis function returns the solution to the equation
+        \n      zk ≡ x (mod y)
+        \nthe result will be k = "solution" (mod y) with "solution" returned
+        \nthis function will return "None" if the equation has no solution, or if 1 or more arguments are not valid
+        \nif z is missing or set to 0 this fuction will return
+        \n        x (mod y)'''
+    
+    try:
+        while True:
+            g = gcd(z, y)
+            if g == 1:
+                if z == 0:
+                    return x % y
+                else:
+                    inverse = 0
+                    z = z % y
+                    x = x % y
+                    for i in range(1, y):
+                        if z*i % y == 1:
+                            inverse = i
+                            break
+                        else:
+                            continue
+                    return int((x*inverse) % y)
+            else:
+                try:
+                    assert int(x/g) == x/g
+                    x /= g
+                    z /= g
+                    y /= g
+                    continue
+                except:
+                    return None
+    except:
+        return None
+
+
+
 # Divisors
 def div(x, y = 1):
     '''\ny is an integer automatically set to 1\nthis function returns the number of divisors of x if y is 1 or missing
@@ -219,30 +220,38 @@ def prime(x):
 
 
 
+# Relatively Prime checker
+def rel(x, y = 0):
+    '''\ny is an integer automatically set to 0
+    \nthis function returns "True" if x and y are relatively prime, it returns False if not, and it returns "None" if 1 or more arguments are not valid
+    \nif y is set to 0 or missing this function will return "True" if x is prime and "False" otherwise'''
+    try:
+        if y == 0:
+            for i in range(2, int(x**(1/2))):
+                if x % i == 0:
+                    return False
+                else:
+                    continue
+            return True
+        else:
+                if gcd(x, y) == 1:
+                    return True
+                else:
+                    return False
+    except:
+        return None
+
+
+
 # Primes less than a given number
 def pi(x, y=1):
     '''\nthis function returns the number of primes less than x if y is 1 or missing\nthis function returns the number of primes between y argument and x otherwise\nit returns "None" if 1 or more arguments are not valid'''
 
     try:
         count = 0
-        def isprime(x):
-            primecheck = 0
-
-            if x == 2:
-                primecheck = 1
-                return primecheck
-            else:
-                for i in range(2, x):
-                    if x % i == 0:
-                        primecheck = 0
-                        return primecheck
-                    else:
-                        primecheck = 1
-            
-            return primecheck
         
         for i in range(y, x):
-            if bool(isprime(i)) == True:
+            if rel(i) == True:
                 count += 1
         return count
     except:
@@ -310,6 +319,21 @@ def col(x):
         return None
 
 
+
+# Euler's Totient Function
+def tot(x):
+    '''\nthis function returns the number of integers between 1 and n that are relatively prime to n\nit returns "None" if the argument is not valid'''
+
+    try:
+        count = 0        
+        for i in range(1, x + 1):
+            if gcd(x, i) == 1:
+                count += 1
+        return count
+    except:
+        return None
+
+
 funcdict = {
     'floor': floor,
     'ceil': ceil,
@@ -324,17 +348,26 @@ funcdict = {
     'primefac': primefac,
     'fac': fac,
     'col': col,
+    'rel': rel,
+    'tot': tot
 }
 
+
+
 # Info
 def info(x = 0):
     '''
     run htam.info() to see general informations about htam functions
     '''
+
+    # All Functions that takes 3 or 2 arguments respectively
+    arg3 = ['mod']
+    arg2 = ['root', 'div', 'gcd', 'lcm', 'pi', 'rel']
+
     if x == 0:
         print(
             '\n' + __pack.upper() + '\n'
-            '\nver. 1.0.4\n '
+            '\nver. 1.1.0\n '
             '\nHere\'s a list of functions ' + __pack + ' can perform. \n'
             )
         for i in funcdict.keys():
@@ -343,10 +376,12 @@ def info(x = 0):
             '\nrun ' + __pack + '.info(*function_name*) to see more detailed informations for that function.'
             )
     elif x in funcdict.keys():
-        if x == 'root' or x == 'div' or x == 'gcd' or x == 'lcm' or x == 'pi':
+        if x in arg2:
             __print_info(x, ',y')
-        elif x == 'mod':
+        elif x in arg3:
             __print_info(x, ', y, z')
         else:
             __print_info(x)
-        print(funcdict[x].__doc__)
\ No newline at end of file
+        print(funcdict[x].__doc__)
+    else:
+        print('No function named', x)
\ No newline at end of file
